#ifndef NAGIOSAGENT_H
#define NAGIOSAGENT_H

#include <QtGui/QWidget>
#include <QSystemTrayIcon>
#include <QMenu>
#include <QAction>
#include <QKeySequence>
#include <QCloseEvent>
#include <QtGui>
#include <QApplication>
#include <QThread>
#include <QTextCodec>
#include <QSettings>
#include <QStringList>
#include <QHttp>
#include <QRegExp>
#include <QList>
#include <QDesktopWidget>
#include <QSslError>
#include <QSslSocket>
#include <QFontMetrics>
#include "ui_nagiosagent.h"
#include "informationwindow.h"

class NagiosAgent : public QWidget
{
    Q_OBJECT
public:
    NagiosAgent(QWidget *parent = 0);
    ~NagiosAgent();
private Q_SLOTS:
    void iconActivated(QSystemTrayIcon::ActivationReason reason);
    void hostsSrvcsMenuHandler(QAction * action);
    void addHost();
    void delHost();
    void addService();
    void delService();
    void checkAndSaveConfig();
    void showAndReadConfig();
    void processHostsFinished(bool error);
    void processSrvcsFinished(bool error);
    void processSslErrors(const QList<QSslError> & sslErrorsList);
    void processTimer();
    bool isConfigOk();
    void httpGetInfo();
    void timerStart();
    void closeAll();
    void showAbout();
    void useHttpsClicked(bool use);
    void trayMessageClicked();
Q_SIGNALS:
	void closeAllInformWin();
private:
	static const int	defaultWidth = 507;
	static const int	defaultHeight = 370;
	static const int 	defaultRequestPeriod = 120; 	//seconds
	static const bool 	defaultAutoClose = false;
	static const bool	useUserDefExecDir = false;
	static const bool	useUserDefHostsRequest = false;
	static const bool	useUserDefSrvcsRequest = false;
	static const bool	useHTTPAuth = false;
	static const bool	defaultAllExceptSpecifedHosts = true;
	static const bool	defaultOnlyListedHosts = !defaultAllExceptSpecifedHosts;
	static const bool	defaultAllExceptSpecifedSrvcs = true;
	static const bool	defaultOnlyListedSrvcs = !defaultAllExceptSpecifedSrvcs;
	static const bool	defaultHostsOnLMK = true;
	static const bool	defaultPopupsStayOnTop = true;
	static const bool	defaultUseHttps = false;
	static const bool	defaultDenyInvalidCert = true;		// SSL   
	static const bool	defaultUseBasicRegExpConfig = true;	//   
	static const bool	defaultUseHostDownStat = true;
	static const bool	defaultUseHostUnreachStat = true;
	static const bool	defaultUseSrvCritStat = true;
	static const bool	defaultUseSrvWarnStat = true;
	static const bool	defaultUseSrvUnknStat = false;
	static const bool	defaultOpenUrlFromTray = true;
	static const int	maxHostsErrors = 4;			//   http    ,
													//    QMessageBox   
	static const int	maxSrvcsErrors = 4;			//    maxHostsErrors   
	static const int	defaultHostStatRxPos = 1;
	static const int	defaultHostLinkRxPos = 2;
	static const int	defaultHostNameRxPos = 3;
	static const int	defaultSrvStatRxPos = 1;
	static const int	defaultSrvLinkRxPos = 2;
	static const int	defaultSrvNameRxPos = 3;
	static const int	maxHttpConnectingTime = 30;	//       
													//        ()
	static const QString defaultHttpServerPort;			//Default HTTP port
	static const QString defaultHttpsServerPort;
	static const QString defaultAutoCloseTime;		//minutes
	static const QString defaultExecDir;
	static const QString defaultHostRequest;
	static const QString defaultSrvcsRequest;
	static const QString defaultBasicHostsRegExp;
	static const QString defaultBasicSrvcsRegExp;
	static const QString defaultExpertHostsRegExp;
	static const QString defaultExpertSrvcsRegExp;
	static const QString trayToolTipHeader;
	static const QString configDirName;
	static const QString programmName;
	struct
	{
		QString serverAddress;
		QString	serverPort;
		int 	requestPeriod;
		bool 	autoClose;
		int 	autoCloseTime;
		int		hostStatRxPos;
		int		hostLinkRxPos;
		int		hostNameRxPos;
		int		srvStatRxPos;
		int		srvLinkRxPos;
		int		srvNameRxPos;
		QString execDir;
		QString	hostsRequest;
		QString	servicesRequest;
		QString hostsRegExp;
		QString srvcsRegExp;
		bool	useUserDefExecDir;
		bool 	useUserDefHostRequest;
		bool	useUserDefSrvcsRequest;
		bool	useHTTPAuth;
		bool	useUserDefHostsRegExp;
		bool	useUserDefSrvcsRegExp;
		bool	allExceptSpecifedHosts;
		bool	onlyListedHosts;
		bool	allExceptSpecifedSrvcs;
		bool	onlyListedSrvcs;
		bool	useTrayMessages;
		bool	disableSounds;
		bool	enableHosts;
		bool	enableSrvcs;
		bool	showHostsStateChanged;
		bool	showSrvcsStateChanged;
		bool	useHttps;
		bool	popupsAlwaysStayOnTop;
		bool	hostsOnLMK;
		bool	denyInvalidCert;
		bool	useBasicRegExpConfig;
		bool	openUrlFromTray;
		QString	httpLogin;
		QString httpPassword;
		QStringList hostsList;
		QStringList	srvcsList;
	}runningConfig;
	enum httpCodes
	{
		httpOk = 200,
		httpMovedPerm = 301,
		httpMovedTemp = 302,
		httpNotFound = 404
	};
	enum tabNums
	{
		commonTab,
		filterHostsTab,
		filterServicesTab,
		statusTab,
		miscTab
	};

    Ui::NagiosAgentClass ui;
	QSystemTrayIcon trayIcon;
	QMenu 	trayMainMenu;
	QMenu 	trayHostsMenu;
	QMenu 	traySrvcsMenu;
	QHttp 	* httpHosts;
	QHttp	* httpSrvcs;
	QTimer 	timerRequests;
	QPoint	informWinPt;
	QTcpSocket	* hostsHttpSocket;
	QTcpSocket	* srvcsHttpSocket;
	QSslSocket	* hostsSslSocket;
	QSslSocket	* srvcsSslSocket;
	int		idGetHosts;
	int		idGetSrvcs;
	int		httpHostsErrors;
	int		httpSrvcsErrors;
	int		httpHostsConnectingTime;
	int		httpSrvcsConnectingTime;
	bool	ignoredHostsSignal;		//   true    (HTTP/HTTPS)
	bool	ignoredSrvcsSignal;		//   true    (HTTP/HTTPS)
	bool	authReqErrorMsgVisible;
	QString	lastUrlForOpenFromTrayMessage;

    void closeEvent(QCloseEvent * event);
    void createTrayIcon();
    void connectSignals();
    void createActions();
    void readConfig();
    void saveConfig();
    void updateRunningConfig();
    void processHostsInfo();
    void processSrvcsInfo();
    void updateInformWinPt();
    void updateTrayToolTip(bool reset = false);
    void updateTrayMenus();
    void reinitQHttp(bool httpsUsed, bool useHttps = false);
    void updateBasicRegExp();
    void showHostUpOrDownMessage(QString hostName, InformationTypes state, QString hostHttpLink,
    		QString hostStatus, QPoint leftCorner);
};

#endif // NAGIOSAGENT_H
